'''

 Filter to implement a SKETCH look of the INPUT IMAGE
 (following a tutorial of Jamack)
 If chosen by the user, 3 different images are produced
 ...................................................... 
 
 Release 1.0   initial
 
Flow implemented to get the final result:      
 
  1. create a new image with dimension chosen by the user
  2. create a BG as a ciopy of the source drawable
  3. duplicate and apply G'MIC Freaky B&W; set mode=Dodge
  4. duplicate, invert and apply Gimp Erode (twice)
  5. create a new layer from visible; set mode=Multiply
  6. duplicate the Bg and set it on top; set mode=Burn (my choice)
  7. create a new layer from visible; set mode=Normal; name it SketchBURN
  8. set visibility off on SketchBURN
  9. change mode on duplicated BG; set it to Color
 10. create a new layer from visible; set mode=Normal; name it SketchCOLOR
 11. set visibility off on SketchCOLOR
 12. change mode on duplicated BG; set it to Overlay
 13. create a new layer from visible; set mode=Normal; name it SketchOVERLAY
 14. if chosen by the user, create and display 3 images, using the 3 layers:
     SketchBURN, SketchCOLOR, SketchOVERLAY 
 ----------------------------------------------------------------
 COPYRIGHT NOTICE
 ----------------------------------------------------------------
 
 This program is free software you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation either version 3 of the License, or
 (at your option) any later version.  
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program if not, you can view the GNU General Public
 License version 3 at the web site http://www.gnu.org/licenses/gpl-3.0.html
 Alternatively you can write to the Free Software Foundation, Inc., 675 Mass
 Ave, Cambridge, MA 02139, USA.
 
 -------------------------------------------------------------------
 other info
 -------------------------------------------------------------------
 the following line terminates the long initial comments
 
'''
#!/usr/bin/env python
from gimpfu import *
import decimal

############################################################################
# include, then use: erro_msg or info_msg
#---------------------------------------------------------------------------
def erro_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(0) # box
    gimp.message ("ERROR: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###
def info_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2) # error console
    gimp.message ("INFO: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###
############################################################################
#

def JamackSketchNoGMIC(inImage, inDrawable, widthheight, doCreate) :

    pdb.gimp_context_push
    currFG = pdb.gimp_context_get_foreground()
    currBG = pdb.gimp_context_get_background()
    
    width = inImage.width
    height = inImage.height
    type = inImage.base_type
    
    name = inDrawable.name
    if '.' in name:
        split = name.split(".", 1)
        name = split[0]
    suffix = ", as a Sketch"
    
    # calculate scale factor for the new output image
    if widthheight == 0 :   # means the user wants to have same size as input
        scale = 1
    else :     
        optiWidthHeight = widthheight       # size to work on to get best results
        realWidthHeight = width+height      # real size of the input image
        scale = decimal.Decimal(optiWidthHeight)/decimal.Decimal(realWidthHeight)      # ratio for scaling

    newwidth=int(width*scale)
    newheight=int(height*scale)
    
    #info_msg ("newwidth="+str(newwidth))
    newImage = pdb.gimp_image_new (newwidth, newheight, type)                    
    newBG = pdb.gimp_layer_new_from_drawable(inDrawable, newImage)     # background layer
    newBG.mode = NORMAL_MODE
    newBG.name = "Background Image Layer"
    newBG.opacity = 100.0 
    newImage.add_layer (newBG, 0)
    pdb.gimp_layer_scale(newBG, newwidth, newheight, False)
    alpha=pdb.gimp_drawable_has_alpha(newBG)
    if (alpha==False):
        pdb.gimp_layer_add_alpha (newBG)    #  RGBA
    
    # create the working layers
    
    FreakyBW = newBG.copy()
    FreakyBW.mode = NORMAL_MODE
    FreakyBW.name = "FreakyBW Layer"
    FreakyBW.opacity = 100.0 
    newImage.add_layer(FreakyBW, 0)
    pdb.gimp_desaturate(FreakyBW)
    
    Inverted = FreakyBW.copy()
    Inverted.mode = DODGE_MODE
    Inverted.name = "Inverted Layer"
    Inverted.opacity = 100.0 
    newImage.add_layer(Inverted, 0)
    pdb.gimp_invert (Inverted)
    pdb.plug_in_erode (newImage, Inverted, 1, HISTOGRAM_VALUE, 1.0, 7, 0, 128)
    pdb.plug_in_erode (newImage, Inverted, 1, HISTOGRAM_VALUE, 1.0, 7, 0, 128)
    
    Visible = pdb.gimp_layer_new_from_visible(newImage, newImage, "Visible Layer")
    newImage.add_layer(Visible, 0)
    pdb.gimp_layer_set_opacity(Visible, 100.0)
    pdb.gimp_layer_set_mode(Visible, MULTIPLY_MODE)
    
    newBG2 = newBG.copy()
    newBG2.mode = BURN_MODE
    newBG2.name = "dup BG Layer"
    newBG2.opacity = 100.0 
    newImage.add_layer(newBG2, 0)
    
    # First Outcome: Burn
    SketchBURN = pdb.gimp_layer_new_from_visible(newImage, newImage, name+suffix+" (Burn)")
    newImage.add_layer(SketchBURN, 0)
    pdb.gimp_layer_set_opacity(SketchBURN, 100.0)
    pdb.gimp_layer_set_mode(SketchBURN, NORMAL_MODE)
    
    # Second Outcome: Color
    pdb.gimp_item_set_visible (SketchBURN, False)
    newBG2.mode = COLOR_MODE
    SketchCOLOR = pdb.gimp_layer_new_from_visible(newImage, newImage, name+suffix+" (Colour)")
    newImage.add_layer(SketchCOLOR, 0)
    pdb.gimp_layer_set_opacity(SketchCOLOR, 100.0)
    pdb.gimp_layer_set_mode(SketchCOLOR, NORMAL_MODE)
    
    # Third Outcome: Overlay
    pdb.gimp_item_set_visible (SketchCOLOR, False)
    newBG2.mode = OVERLAY_MODE
    SketchOVERLAY = pdb.gimp_layer_new_from_visible(newImage, newImage, name+suffix+" (Overlay)")
    newImage.add_layer(SketchOVERLAY, 0)
    pdb.gimp_layer_set_opacity(SketchOVERLAY, 100.0)
    pdb.gimp_layer_set_mode(SketchCOLOR, NORMAL_MODE)

    
    if doCreate == True :
        # first new image: burn
        pdb.gimp_item_set_visible (SketchBURN, True)
        BurnImage = pdb.gimp_image_new (newwidth, newheight, type)                    
        BurnBG = pdb.gimp_layer_new_from_drawable(SketchBURN, BurnImage)     # background layer
        BurnBG.mode = NORMAL_MODE
        BurnBG.name = SketchBURN.name
        BurnBG.opacity = 100.0 
        BurnImage.add_layer (BurnBG, 0)
        pdb.gimp_display_new (BurnImage)
        # second new image: color
        pdb.gimp_item_set_visible (SketchCOLOR, True)
        ColorImage = pdb.gimp_image_new (newwidth, newheight, type)                    
        ColorBG = pdb.gimp_layer_new_from_drawable(SketchCOLOR, ColorImage)     # background layer
        ColorBG.mode = NORMAL_MODE
        ColorBG.name = SketchCOLOR.name
        ColorBG.opacity = 100.0 
        ColorImage.add_layer (ColorBG, 0)
        pdb.gimp_display_new (ColorImage)
        # first new image: burn
        pdb.gimp_item_set_visible (SketchOVERLAY, True)
        OverlayImage = pdb.gimp_image_new (newwidth, newheight, type)                    
        OverlayBG = pdb.gimp_layer_new_from_drawable(SketchOVERLAY, OverlayImage)     # background layer
        OverlayBG.mode = NORMAL_MODE
        OverlayBG.name = SketchOVERLAY.name
        OverlayBG.opacity = 100.0 
        OverlayImage.add_layer (OverlayBG, 0)
        pdb.gimp_display_new (OverlayImage)
    
    pdb.gimp_display_new (newImage)
    pdb.gimp_displays_flush
   
    # Close the undo group.
    pdb.gimp_context_pop
    pdb.gimp_context_set_foreground(currFG)
    pdb.gimp_context_set_background(currBG)
    
    return(SketchBURN)

# This is the plugin registration function
register(
    "JamackSketchNoGMIC",    
    "to set a sketch lookin 3 different modes on an image",   
    "This script sets a sketch lookin 3 different modes on an image (Jamack tgut).",
    "Diego", 
    "Diego Nassetti ", 
    "October 2016",
    "JamackSketchNoGMIC", 
    "RGB*", 
    [
      (PF_IMAGE, "image", "Input image", None),
      (PF_DRAWABLE, "idrawable", "Input drawable", None),
      (PF_ADJUSTMENT, "size", "Width+Height of New Image (0=same as input)", 0, (0, 9600, 300)),
      (PF_TOGGLE, "create3", "Create 3 Images?", False),
    ], 
    [
      (PF_DRAWABLE, "odrawable", "Output drawable", None),  
    ],
    JamackSketchNoGMIC,
    menu="<Image>/Diego/Artistic",
    )

main()
